#!/usr/bin/env bash

# Verify if anything has changed and a new MR was required.
STATUS=$(git status)
TEST='nothing to commit, working tree clean'
if [[ "$STATUS" == *"${TEST}"* ]]; then
  echo "No changes detected, no MR required";
  exit 0
fi

# Look which is the default branch
TARGET_BRANCH=${CI_COMMIT_REF_NAME}

# Determine the source branch
SOURCE_BRANCH=${TARGET_BRANCH}-proposed-update-$(date +%Y-%m-%d-%H-%M-%S)

# Prepare the URL to push to
[[ $HOST =~ (^https?://)(.+) ]] && REPO="${BASH_REMATCH[1]}${GITLAB_USER_LOGIN}:${ACCESS_TOKEN}@${BASH_REMATCH[2]}.git"
# Extract the host where the server is running, and add the URL to the APIs
[[ $HOST =~ ^https?://[^/]+ ]] && HOST="${BASH_REMATCH[0]}/api/v4/projects/"

# The description of our new MR, we want to remove the branch after the MR has been closed
BODY="{
    \"id\": ${CI_PROJECT_ID},
    \"source_branch\": \"${SOURCE_BRANCH}\",
    \"target_branch\": \"${TARGET_BRANCH}\",
    \"remove_source_branch\": true,
    \"title\": \"WIP: ${SOURCE_BRANCH}\",
    \"assignee_id\":\"${GITLAB_USER_ID}\"
}";

# Create new branch and push changes upstream
git config --global user.email "${GITLAB_USER_EMAIL}"
git config --global user.name "${GITLAB_USER_NAME}"
git remote add mr ${REPO}
git fetch mr
git branch ${SOURCE_BRANCH}
git checkout ${SOURCE_BRANCH}
git commit -am "GitLab CI proposed changes"
git push -u mr ${SOURCE_BRANCH}

# Create MR
curl --fail --retry 2 -X POST "${HOST}${CI_PROJECT_ID}/merge_requests" \
    --header "PRIVATE-TOKEN:${ACCESS_TOKEN}" \
    --header "Content-Type: application/json" \
    --data "${BODY}";

echo "Opened a new merge request: WIP: ${SOURCE_BRANCH} and assigned to you";
